
% REPRODUCING EXPERIMENT 1

clear; % we need to clear the variables in workspace before learning. 
load 'nlf'; 
mex Ctransform.c; 
mex Clearn.c;   % compile C code

% CONSTANTS %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
scale = .7;  % scale of Gabors, length = 17 pixels
norient = 15;  % number of orientations
epsilon = .1;  % allowed correlation between selected Gabors
T = 0.;        % threshold for responses

sub = 2;    % subsampling rate for Gabor
Lrange = 3; % allowed shifting in location (times sub pixels)
Orange = 1; % allowed shifting in orientation
%%% Note: Lrange*sub is the pixels allowed to shift along normal
%%%       Lrange*sub should be the same in learning and testing

LIK = 1.; SHUTUP = 0.;  % use log-likelihood score. If LIK=0, then use active correlation
% SHUTUP is to neglect the positions with low average responses
                       
% LIK = 0; SHUTUP = 1.;   



nstep = 3;  % number of iterations
LIK = 1.;  % use log-likelihood score. If LIK=0, then use active correlation
%LIK = 0.;  


binnum = 50; % number of bins 



[allfilter, allsymbol] = makefilter(scale, norient);  % generate Gabor filters 
h = (size(allfilter{1}, 1)-1)/2;  % half size of Gabor
C = corr(allfilter, epsilon);    % generate the inhibition maps


% TRAINING IMAGES, only the first one needs to be labeled with bounding box
CAT=0; DEER=0; PIGEON=0; SWAN=0; WOLF=0; HORSE=1;  

if (CAT>.5)
Iname = {'000881.jpg' '000749.jpg' '000958.jpg' '001040.jpg' '000464.jpg'};
Totalsketch = 60;  % allowed number of selected Gabors   
fold = 1.; 
sx0 = 136; sy0 = 140;  % size of bounding box
xhere = 4;  yhere = 42; ihere = 0;  % position of the bounding box in image ihere
sx0 = floor(sx0*fold+.5); sy0 = floor(sy0*fold+.5); 
xhere = floor(xhere*fold+.5); yhere = floor(yhere*fold+.5); 
end

if (DEER>.5)
Iname = {'dear1.jpg' 'dear2.jpg' 'dear3.jpg' 'dear4.jpg' 'dear5.jpg'};
Totalsketch = 60;  % allowed number of selected Gabors   
sx0 = 168; sy0 = 185;  % size of bounding box
xhere = 22;  yhere = 45; ihere = 0;  % position of the bounding box in image ihere
fold = .8; 
sx0 = floor(sx0*fold+.5); sy0 = floor(sy0*fold+.5); 
xhere = floor(xhere*fold+.5); yhere = floor(yhere*fold+.5); 
end

if (PIGEON>.5)
Iname = {'pigeo1.jpg' 'pigeo2.jpg' 'pigeo3.jpg' 'pigeo4.jpg' 'pigeo5.jpg'};
Totalsketch = 50;  % allowed number of selected Gabors   
sx0 = 157; sy0 = 165;  % size of bounding box
fold = .7; 
xhere = 16;  yhere = 65; ihere = 0;  % position of the bounding box in image ihere
sx0 = floor(sx0*fold+.5); sy0 = floor(sy0*fold+.5); 
xhere = floor(xhere*fold+.5); yhere = floor(yhere*fold+.5); 
end

if (SWAN>.5)
fold = .7; 
Totalsketch = 50;  % allowed number of selected Gabors   
% TRAINING IMAGES, only the first one needs to be labeled with bounding box
Iname = {'swan1.jpg' 'swan2.jpg' 'swan3.jpg' 'swan4.jpg' 'swan5.jpg'};
sx0 = 187; sy0 = 257;  % size of bounding box
xhere = 23;  yhere = 27; ihere = 0;  % position of the bounding box in image ihere
sx0 = sx0 - xhere; sy0 = sy0 - yhere; 
sx0 = floor(sx0*fold+.5); sy0 = floor(sy0*fold+.5); 
xhere = floor(xhere*fold+.5); yhere = floor(yhere*fold+.5); 
end

if (WOLF>.5)
fold = .6; 
Totalsketch = 50;  % allowed number of selected Gabors   
% TRAINING IMAGES, only the first one needs to be labeled with bounding box
Iname = {'wol1.jpg' 'wol2.jpg' 'wol3.jpg' 'wol4.jpg' 'wol5.jpg'};
sx0 = 199; sy0 = 208;  % size of bounding box
xhere = 6;  yhere = 14; ihere = 0;  % position of the bounding box in image ihere
sx0 = sx0 - xhere; sy0 = sy0 - yhere; 
sx0 = floor(sx0*fold+.5); sy0 = floor(sy0*fold+.5); 
xhere = floor(xhere*fold+.5); yhere = floor(yhere*fold+.5); 
end

if (HORSE>.5)
fold = .8; 
Totalsketch = 60;  % allowed number of selected Gabors   
% TRAINING IMAGES, only the first one needs to be labeled with bounding box
Iname = {'hors1.jpg' 'hors2.jpg' 'hors3.jpg' 'hors4.jpg' 'hors5.jpg'};
sx0 = 131; sy0 = 187;  % size of bounding box
xhere = 14;  yhere = 15; ihere = 0;  % position of the bounding box in image ihere
sx0 = sx0 - xhere; sy0 = sy0 - yhere; 
sx0 = floor(sx0*fold+.5); sy0 = floor(sy0*fold+.5); 
xhere = floor(xhere*fold+.5); yhere = floor(yhere*fold+.5); 
end

nimage = size(Iname, 2);
Sx = zeros(1, nimage); Sy = zeros(1, nimage); 
for (i = 1 : nimage)
    I{i} = randn(sx0, sy0); 
    J{i} = imresize(double(rgb2gray(imread(Iname{i}))), fold, 'nearest'); 
    [sx1, sy1] = size(J{i}); 
    Sx(i) = sx1; Sy(i) = sy1; % sizes of images
end

% VARIABLES FOR MEX-C %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
Mi = zeros(1, Totalsketch);  % orientation of selected Gabor
Mx = zeros(1, Totalsketch);  % position of selected Gabor
My = zeros(1, Totalsketch); 
Mm = zeros(1, Totalsketch);  % lambda
Mm1 = zeros(1, Totalsketch);  % logZ
sym = zeros(size(I{1}));  % symbolic plot of selected Gabor
Asym = cell(1, nimage);   % symbolic plot for each image with active Gabor
for (i = 1 : nimage)
    Asym{i} = zeros(size(I{1}));  
end
tss = zeros(nimage, 1);  % total scores for testing images
gain = zeros(Totalsketch, 1);  % coding gain for selected Gabor

J0 = J{1}; 
I0{1} = J0(xhere:(xhere+sx0-1), yhere:(yhere+sy0-1)); 
nimage0 = 1; Lrange0 = 0; Orange0 = 0; 

[allfiltered0, allmax0] = applyfilter(I0, allfilter); 

Asym0{1} = zeros(size(I{1}));
tss0 = zeros(1, 1); 

Ctransform(nimage0, norient, allfiltered0, sx0, sy0, Upperbound);
'learning'
tic
Clearn(nimage0, norient, allfiltered0, C, h, sx0, sy0, T, Mi, Mx, My, Mm, Mm1, allsymbol(1, :), sym, Asym0, Lrange0, Orange0, sub, Totalsketch, Upperbound, tss0, gain, SHUTUP, binnum, M, lam, e, lz);
disp(['mex-C learning time: ' num2str(toc) ' seconds']);

figure; 
imshow(-sym, []); 




mex Cfind.c;   % compile C code
mex Ccopy.c; 
mex Ctransform1.c; 
totalScale = 9;  % total number of scales
ax = floor(sx0/2); ay = floor(sy0/2); % half size of bounding box
sub1 = 1; Lrange1 = 6; % we scan every pixel, Lrange*sub is fixed in learning and testing

Sxx = zeros(nimage, totalScale); Sxa = zeros(1, totalScale*nimage); 
Syy = zeros(nimage, totalScale); Sya = zeros(1, totalScale*nimage); 
mscale = zeros(nimage, totalScale);  % log-likelihood score over scales
JJ = cell(1, totalScale*nimage); 
tss1 = cell(1, totalScale*nimage); 
sym1 = cell(1, totalScale*nimage); 

for (i = 1:nimage)
  for(j=1:totalScale)
    JJ{(i-1)*totalScale+j} = imresize(J{i}, .8+(j-1)*.05, 'nearest');  % scaled images
    [sx1, sy1] = size(JJ{(i-1)*totalScale+j}); 
    Sxx(i, j) = sx1; Syy(i, j) = sy1; % sizes of images
    Sxa((i-1)*totalScale+j) = sx1; Sya((i-1)*totalScale+j) = sy1; 
    tss1{(i-1)*totalScale+j} = zeros(sx1, sy1);  % log-likelihood score map for testing images
    sym1{(i-1)*totalScale+j} = zeros(sx1, sy1);  % symbolic sketch on testing image
  end
end
[allfiltered1, allmaxed] = applyfilter(JJ, allfilter);  % filter training images

% VARIABLES %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


% WHITENING %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% the following three lines are the only things that need to be added



Ctransform1(totalScale*nimage, norient, allfiltered1, Sxa, Sya, Upperbound);


for(it = 1:3)
 for (i = 1:nimage)
  for(j=1:totalScale)
    [sx1, sy1] = size(JJ{(i-1)*totalScale+j}); 
    tss1{(i-1)*totalScale+j} = zeros(sx1, sy1);  % log-likelihood score map for testing images
    sym1{(i-1)*totalScale+j} = zeros(sx1, sy1);  % symbolic sketch on testing image
  end

% MEX C %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
'finding'
msci = zeros(1, totalScale); Fx = zeros(1, totalScale); Fy = zeros(1, totalScale); 
Sxi = Sxx(i, :); Syi = Syy(i, :); 
Cfind(i, nimage, totalScale, norient, allfiltered1, allmaxed, C, h, ax, ay, T, Mi, Mx, My, Mm, Mm1, allsymbol(1, :), Lrange1, Orange, sub1, Totalsketch, Upperbound, tss1, sym1, Sxi, Syi, msci, Fx, Fy);

% RESULTS %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%    


maxscale = max(msci);   % most likely scale
for(j=1:totalScale)
    if(msci(j)==maxscale)
       figure; 
       subplot(2, 2, 1, 'v6');  imshow(J{i}, []);   % original image
       subplot(2, 2, 2, 'v6');  imshow(JJ{(i-1)*totalScale+j}+sym1{(i-1)*totalScale+j}*100, []);  % overlaid sketch 
       subplot(2, 2, 3, 'v6');  plot(msci);       % log-likelihood over scale      
       subplot(2, 2, 4, 'v6');  imshow(-tss1{(i-1)*totalScale+j}, []);  % log-likelihood score map
       'copying'
       Ccopy(I{i}, JJ{(i-1)*totalScale+j}, floor(Fx(j)+.5), floor(Fy(j)+.5), sx0, sy0, Sxi(j), Syi(j)); 
    end
end
end


Mi = zeros(1, Totalsketch);  % orientation of selected Gabor
Mx = zeros(1, Totalsketch);  % position of selected Gabor
My = zeros(1, Totalsketch); 
Mm = zeros(1, Totalsketch);  % lambda
Mm1 = zeros(1, Totalsketch);  % logZ
sym = zeros(size(I{1}));  % symbolic plot of selected Gabor
Asym = cell(1, nimage);   % symbolic plot for each image with active Gabor
for (i = 1 : nimage)
    Asym{i} = zeros(size(I{1}));  
end
tss = zeros(nimage, 1);  % total scores for testing images
gain = zeros(Totalsketch, 1);  % coding gain for selected Gabor

[allfiltered, allmaxed00] = applyfilter(I, allfilter); 
Ctransform(nimage, norient, allfiltered, sx0, sy0, Upperbound);
'learning'
tic
Clearn(nimage, norient, allfiltered, C, h, sx0, sy0, T, Mi, Mx, My, Mm, Mm1, allsymbol(1, :), sym, Asym, Lrange, Orange, sub, Totalsketch, Upperbound, tss, gain, SHUTUP, binnum, M, lam, e, lz);
disp(['mex-C learning time: ' num2str(toc) ' seconds']);

figure; 
imshow(-sym, []); 
end
